﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Random;

namespace HeuristicLab.BioBoost.Operators.Mutation {

  [StorableClass]
  public class PlantSupplierUtilizationExchanger : PlantBasedUtilizationModifier {

    #region Construction & Cloning
    [StorableConstructor]
    public PlantSupplierUtilizationExchanger(bool isDeserializing) {}
    public PlantSupplierUtilizationExchanger(PlantSupplierUtilizationExchanger orig, Cloner cloner) : base(orig, cloner) {}
    public PlantSupplierUtilizationExchanger() {}
    public override IDeepCloneable Clone(Cloner cloner) {
      return new PlantSupplierUtilizationExchanger(this, cloner);
    }
    #endregion

    public override void Redistribute(RealVector utilizations, List<int> supplierIndices) {
      var suppliers = supplierIndices.SampleRandom(Random, 2).ToArray();
      if (suppliers.Length >= 2) {
        var temp = utilizations[suppliers[0]];
        utilizations[suppliers[0]] = utilizations[suppliers[1]];
        utilizations[suppliers[1]] = temp;
      }
    }


  }
}
